@@ -9,18 +9,20 @@ use FindBin qw($Bin);
 use lib "$Bin/../lib";
 use Statistics::Descriptive::LogScale;
 
+my $load;
 my %opt = (width => 600, height => 200, trim => 0);
 
 # Don't require module just in case
 GetOptions (
-	'base=s' => \$opt{base},
-	'floor=s' => \$opt{zero},
-	'width=s' => \$opt{width},
-	'height=s' => \$opt{height},
+	'b|base=s' => \$opt{base},
+	'f|floor=s' => \$opt{zero},
+	'w|width=s' => \$opt{width},
+	'h|height=s' => \$opt{height},
 	'ltrim=s' => \$opt{ltrim},
 	'utrim=s' => \$opt{utrim},
 	'min=s' => \$opt{min},
 	'max=s' => \$opt{max},
+	'l|load=s' => \$load,
 	'help' => \&usage,
 ) or usage();
 
@@ -37,6 +39,7 @@ Options may include:
   --utrim <0..100> - cut that % off from the right
   --min <xxx> - strip data below this value
   --max <xxx> - strip data above this value
+  --load, -l - load data from a JSON file
   --help - this message
 EOF
 	exit 2;
@@ -56,11 +59,22 @@ if ($out eq '-') {
 # sane default for precision = 1 pixel at right/left side
 $opt{base} = 1+1/$opt{width} unless defined $opt{base};
 
-my $stat = Statistics::Descriptive::LogScale->new(
-	base => $opt{base}, linear_width => $opt{zero});
-
-while (<STDIN>) {
-	$stat->add_data(/(-?\d+(?:\.\d*)?)/g);
+my $stat;
+if (defined $load) {
+	eval { require JSON::XS; 1 }
+		or die "JSON::XS is required for --load option to work\n";
+	local $/;
+	open (my $fd, "<", $load) or die "Failed to r-open $load: $!";
+	defined (my $json = <$fd>) or die "Failed to read from $load: $!";
+	close $fd;
+	my $raw = JSON::XS::decode_json($json);
+	$stat = Statistics::Descriptive::LogScale->new(%$raw);
+} else {
+	$stat = Statistics::Descriptive::LogScale->new(
+		base => $opt{base}, linear_width => $opt{zero});
+	while (<STDIN>) {
+		$stat->add_data(/(-?\d+(?:\.\d*)?)/g);
+	};
 };
 
 if (!$stat->count) {
@@ -68,35 +82,36 @@ if (!$stat->count) {
 	exit 3;
 };
 
+# Let's do the real work
 my ($width, $height) = @opt{"width", "height"};
 my $hist = $stat->histogram( %opt, count => $width);
 
-my $trimmer = Statistics::Descriptive::LogScale->new;
-$trimmer->add_data(map { $_->[0]} @$hist);
-
-my $max_val = $trimmer->percentile(99)/0.7;
+# Find maximum or heights
+my $max_val = $hist->[0][0];
+$_->[0] > $max_val and $max_val = $_->[0] for @$hist;
 $_->[0] /= $max_val for @$hist;
 
-# warn "hist = @hist\n";
 # draw!
 my $gd = GD::Simple->new($width, $height);
 $gd->bgcolor('white');
 $gd->clear;
 
+# draw scale
 my $scale = 10;
-
 foreach (1 .. ($scale-1)) {
 	$gd->fgcolor( 'blue' );
 	$gd->line( $width * ($_/$scale), 0, $width * ($_/$scale), $height );
 };
 
+# plot data
 my $i=0;
 foreach (@$hist) {
-	$gd->fgcolor( $_->[0] > 1 ? 'red' : 'orange');
+	$gd->fgcolor( 'orange');
 	$gd->line($i, $height, $i, $height*(1-$_->[0]));
 	$i++;
 };
 
+# finally, print digits (do it AFTER the plotting)
 my $min = $hist->[0][1];
 my $max = $hist->[-1][2];
 my $range = $max - $min;
@@ -109,4 +124,5 @@ foreach (0 .. ($scale-1)) {
 	$gd->string( sprintf("%0.1f", $min + $range*$_/$scale) );
 };
 
+# all folks
 print $fd $gd->png;
