@@ -11,14 +11,16 @@ statistics class.
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
-our $VERSION = 0.0701;
+our $VERSION = 0.08;
 
 =head1 SYNOPSIS
 
+=head2 Basic usage
+
 The basic usage is roughly the same as that of L<Statistics::Descriptive::Full>.
 
     use Statistics::Descriptive::LogScale;
@@ -32,7 +34,68 @@ The basic usage is roughly the same as that of L<Statistics::Descriptive::Full>.
     # This can also be done in O(1) memory, precisely
     printf "Mean: %f +- %f\n", $stat->mean, $stat->standard_deviation;
     # This requires storing actual data, or approximating
+    printf "25%%  : %f\n", $stat->percentile(25);
     printf "Median: %f\n", $stat->median;
+    printf "75%%  : %f\n", $stat->percentile(75);
+
+=head2 Save/load
+
+This is not present in L<Statistics::Descriptive::Full>.
+The save/load interface is designed compatible with JSON::XS.
+However, any other serializer can be used.
+The C<TO_JSON> method is I<guaranteed> to return unblessed hashref
+with enough information to restore the original object.
+
+    use Statistics::Descriptive::LogScale;
+    my $stat = Statistics::Descriptive::LogScale->new ();
+
+    # ..... much later
+    # Save
+    print $fd encoder_of_choice( $stat->TO_JSON )
+        or die "Failed to save: $!";
+
+    # ..... and even later
+    # Load
+    my $plain_hash = decoder_of_choice( $raw_data );
+    my $copy_of_stat = Statistics::Descriptive::LogScale->new( %$plain_hash );
+
+    # Import into existing LogScale instance
+    my $plain_hash = decoder_of_choice( $more_raw_data );
+    $copy_of_stat->add_data_hash( $more_raw_data->{data} );
+
+=head2 Histograms
+
+Both L<Statistics::Descriptive::Full> and L<Statistics::Descriptive::LogScale>
+offer C<frequency_distribution_ref> method for querying data point counts.
+However, there's also C<histogram> method for making pretty pictures.
+Here's a simple text-based histogram.
+A proper GD example was too long to fit into this margin.
+
+    use strict;
+    use warnings;
+
+    use Statistics::Descriptive::LogScale;
+    my $stat = Statistics::Descriptive::LogScale->new ();
+
+    # collect/load data ...
+    my $re_float = qr([-+]?(?:\d+\.?\d*|\.\d+)(?:[Ee][-+]?\d+)?);
+    while (<>) {
+        $stat->add_data($_) for m/($re_float)/g;
+    };
+    die "Empty set"
+        unless $stat->count;
+
+    # get data in [ count, lower_bound, upper_bound ] format as arrayref
+    my $hist = $stat->histogram( count => 20 );
+
+    # find maximum value to use as a scale factor
+    my $scale = $hist->[0][0];
+    $scale < $_->[0] and $scale = $_->[0] for @$hist;
+
+    foreach (@$hist) {
+        printf "%10f %s\n", $_->[1], '#' x ($_->[0] * 68 / $scale);
+    };
+    printf "%10f\n", $hist->[-1][2];
 
 =head1 DESCRIPTION
 
@@ -81,8 +144,8 @@ my $inf = 9**9**9;
 
 =over
 
-=item * base - ratio of adjacent bins. Default is 10^(1/48), which gives
-5% precision and exact decimal powers.
+=item * base - ratio of adjacent bins. Default is 10^(1/232), which gives
+1% precision and exact decimal powers.
 This value represents acceptable relative error in analysis results.
 
 B<NOTE> Actual value may be slightly less than requested one.
@@ -119,10 +182,10 @@ sub new {
 	my $class = shift;
 	my %opt = @_;
 
-	# base for logarithmic bins, use sane default (~5%) if none given
+	# base for logarithmic bins, sane default: +-1%, exact decimal powers
 	# UGLY HACK number->string->number to avoid
 	#     future serialization inconsistencies
-	$opt{base} ||= 10**(1/48);
+	$opt{base} ||= 10**(1/232);
 	$opt{base} = 0 + "$opt{base}";
 	$opt{base} > 1 or croak __PACKAGE__.": new(): base must be >1";
 
@@ -179,6 +242,13 @@ These methods are used to query the distribution properties. They generally
 follow the interface of L<Statistics::Descriptive> and co,
 with minor additions.
 
+All methods return C<undef> on empty data set, except for
+C<count>, C<sum>, C<sumsq> and C<variance> which all return 0.
+
+B<NOTE> This module caches whatever it calculates very agressively.
+Don't hesitate to use statistical functions (except for sum_of/mean_of)
+more than once. The cache is deleted upon data entry.
+
 =head2 clear
 
 Destroy all stored data.
@@ -197,17 +267,25 @@ sub clear {
 
 Add numbers to the data pool.
 
+Returns self, so that methods can be chained.
+
+If incorrect data is given (i.e. non-numeric, undef),
+an exception is thrown and only partial data gets inserted.
+The state of object is guaranteed to remain consistent in such case.
+
+B<NOTE> Cache is reset, even if no data was actually inserted.
+
 =cut
 
 sub add_data {
 	my $self = shift;
-	return unless @_;
 
 	delete $self->{cache};
 	foreach (@_) {
-		$self->{count}++;
 		$self->{data}{ $self->_round($_) }++;
+		$self->{count}++;
 	};
+	$self;
 };
 
 =head2 count
@@ -250,7 +328,7 @@ Return sample range of the dataset, i.e. max() - min().
 
 sub sample_range {
 	my $self = shift;
-	return $self->max - $self->min;
+	return $self->count ? $self->max - $self->min : undef;
 };
 
 =head2 sum
@@ -277,7 +355,7 @@ sub sumsq {
 
 =head2 mean
 
-Return mean, which is sum()/count().
+Return mean, or average value, i.e. sum()/count().
 
 =cut
 
@@ -323,20 +401,24 @@ sub standard_deviation {
 =head2 cdf ($x)
 
 Cumulative distribution function. Returns estimated probability of
-random data point from the sample being less than $x.
+random data point from the sample being less than C<$x>.
 
-As a special case, cdf(0) accounts for I<half> of zeroth bin count (if any).
+As a special case, C<cdf(0)> accounts for I<half> of zeroth bin count (if any).
 
 Not present in Statistics::Descriptive::Full, but appears in
 L<Statistics::Descriptive::Weighted>.
 
+=head2 cdf ($x, $y)
+
+Returns probability of a value being between C<$x> and C<$y> ($x <= $y).
+This is essentially C<cdf($y)-cdf($x)>.
+
 =cut
 
 sub cdf {
 	my $self = shift;
-	my $x = shift;
 	return unless $self->{count};
-	return $self->_count($x) / $self->{count};
+	return $self->_count(@_) / $self->{count};
 };
 
 =head2 percentile( $n )
@@ -347,6 +429,7 @@ Find $n-th percentile, i.e. a value below which lies $n % of the data.
 (see Statistics::Descriptive).
 
 $n is a real number, not necessarily integer.
+
 =cut
 
 sub percentile {
@@ -457,6 +540,7 @@ Dies unless all data points are of the same sign.
 sub geometric_mean {
 	my $self = shift;
 
+	return unless $self->count;
 	croak __PACKAGE__.": geometric_mean() called on mixed sign sample"
 		if $self->min * $self->max < 0;
 
@@ -545,78 +629,37 @@ sub std_moment {
 =head2 mode
 
 Mode of a distribution is the most common value for a discrete distribution,
-or maximum of probability density for continuous one. We assume the
-distribution IS continuous, as we're already approximating.
+or maximum of probability density for continuous one.
 
-So we count probability density by smoothing hit counts in nearest nonempty
-intervals to stabilize it a little.
+For now we assume that the distribution IS discrete, and return the bin with
+the biggest hit count.
 
-NOTE A better algorithm is wanted. Experimental.
-
-NOTE Testing shows mode fairly unstable around zero, e.g.
-normal distribution (10,10) returns mode close to 0.
+NOTE A better algorithm is still wanted. Experimental.
+Behavior may change in the future.
 
 =cut
 
+# Naive implementation
+# Find bin w/greatest count and return it
 sub mode {
 	my $self = shift;
 
+	return if !$self->count;
+
 	my $index = $self->_sort;
-	return if (!@$index);
 	return $index->[0] if @$index == 1;
 
-	my @mode = (0, undef); # [ max_density, bucket_index ]
-	my $density = $self->_probability_density;
+	my @count = map { $self->{data}{$_} } @$index;
 
-	die "Uhhuh" unless @$index == @$density;
-
-	for my $i (0..@$index-1) {
-		$density->[$i] > $mode[0] or next;
-		$mode[0] = $density->[$i];
-		$mode[1] = $index->[$i]
+	my $max_index;
+	my $max_growth = 0;
+	for (my $i = 0; $i<@count; $i++) {
+		$count[$i] > $max_growth or next;
+		$max_index = $i;
+		$max_growth = $count[$i];
 	};
 
-	return $mode[1];
-};
-
-sub _probability_density {
-	my $self = shift;
-
-	my $index = $self->_sort;
-	return [] unless @$index >= 2;
-
-	# FIXME AWFUL
-	# We cannot calculate mode by comparing bin counts:
-	#   bins differ in size, and wide ones would naturally
-	#   contain more hits.
-	# However, simple division by bin size would result in instability
-	#   around zero. Besides, zeroth bin may have zero width.
-	# So, we add up adjacent nonempty bins to stabilize the
-	#   damned thing a little.
-	#                   C[prev] + 2 * C[this] + C[next]
-	# As in, density = ---------------------------------
-	#                          2 * |next - prev|
-	# The egde bins get zero instead of right/left partner,
-	#   because life's so unfair.
-	# Still I fear it's hacky. I wish I knew better.
-	# Mode was a hell to implement.
-
-	my $bin = $self->{data};
-	my @density;
-
-	for (my $i = 1; $i < @$index-1; $i++ ) {
-		my $count = $bin->{ $index->[$i] }
-			+ ($bin->{ $index->[$i-1] }+$bin->{ $index->[$i+1] })/2;
-		$density[$i] = $count / ($index->[$i+1] - $index->[$i-1]);
-	};
-	# special cases
-	my $count = 0 + $bin->{ $index->[0] } + $bin->{ $index->[1] } / 2;
-	$density[0] = $count / (($index->[1] - $index->[0]) * 2);
-
-	$count = $bin->{ $index->[-2] } / 2 + $bin->{ $index->[-1] } + 0;
-	push @density, $count / (($index->[-1] - $index->[-2]) * 2);
-
-	return \@density;
+	return $index->[$max_index];
 };
 
 =head2 frequency_distribution_ref( \@index )
@@ -638,6 +681,7 @@ sub frequency_distribution_ref {
 	my $self = shift;
 	my $index = shift;
 
+	return unless $self->count;
 	# ah, compatibility - return last value
 	return $self->{cache}{frequency_distribution_ref}
 		unless defined $index;
@@ -720,6 +764,17 @@ sub linear_threshold {
 Add values with weights. This can be used to import data from other
 Statistics::Descriptive::LogScale object.
 
+Negative counts are treated as "forgetting" data.
+If a bin count goes below zero, such bin is simply discarded.
+Count is guaranteed to remain consistent in such case.
+
+Returns self, so that methods can be chained.
+
+If incorrect data is given (i.e. non-numeric, undef),
+an exception is thrown and nothing gets inserted.
+
+B<NOTE> Cache is reset, even if no data was actually inserted.
+
 =cut
 
 sub add_data_hash {
@@ -727,10 +782,30 @@ sub add_data_hash {
 	my $hash = shift;
 
 	delete $self->{cache};
-	foreach (keys %$hash) {
-		$self->{count} += $hash->{$_};
-		$self->{data}{ $self->_round($_) } += $hash->{$_};
+
+	# check incoming data for consistency
+	eval {
+		use warnings FATAL => qw(numeric);
+		no warnings qw(void);
+		defined $_ and 0+$_ for %$hash;
 	};
+	croak __PACKAGE__.": add_data_hash failed: $@"
+		if $@;
+
+	# update our counters
+	foreach (keys %$hash) {
+		next unless $hash->{$_};
+		my $key = $self->_round($_);
+		$self->{data}{$key} += $hash->{$_};
+		if ($self->{data}{$key} > 0) {
+			# normal insert
+			$self->{count} += $hash->{$_};
+		} else {
+			# erase empty bin, adjust count
+			$self->{count} += $hash->{$_} - delete $self->{data}{$key};
+		};
+	};
+	$self;
 };
 
 =head2 get_data_hash
@@ -923,9 +998,9 @@ Options may include:
 
 =item * index (+) - interval borders as array. Will be sorted before processing.
 
-=item * min - ignore values below this. default = max + epsilon.
+=item * min - ignore values below this. Default = $self->min - epsilon.
 
-=item * max - ignore values above this. default = min - epsilon.
+=item * max - ignore values above this. Default = $self->max + epsilon.
 
 =item * ltrim - ignore this % of values on lower end.
 
@@ -944,6 +1019,7 @@ sub histogram {
 	my $self = shift;
 	my %opt = @_;
 
+	return unless $self->count;
 	my ($min, $max) = $self->find_boundaries( %opt );
 	# build/check index
 	my @index = @{ $opt{index} || [] };
@@ -997,6 +1073,9 @@ returned values.
 sub find_boundaries {
 	my $self = shift;
 	my %opt = @_;
+
+	return unless $self->count;
+
 	# preprocess boundaries
 	my $min = defined $opt{min} ? $opt{min} : $self->_lower( $self->min );
 	my $max = defined $opt{max} ? $opt{max} : $self->_upper( $self->max );
@@ -1141,6 +1220,8 @@ sub _round {
 	my $self = shift;
 	my $x = shift;
 
+	use warnings FATAL => qw(numeric uninitialized);
+
 	if (abs($x) <= $self->{linear_thresh}) {
 		return $self->{linear_width}
 			&& $self->{linear_width} * floor( $x / $self->{linear_width} + 0.5 );
@@ -1155,6 +1236,8 @@ sub _lower {
 	my $self = shift;
 	my $x = shift;
 
+	use warnings FATAL => qw(numeric uninitialized);
+
 	if (abs($x) <= $self->{linear_thresh}) {
 		return $self->{linear_width}
 			&& $self->{linear_width} * (floor( $x / $self->{linear_width} + 0.5) - 0.5);
